#Program na  mereni Hallova jevu na ruznych teplotach se stabilizaci na dane teplote
#pouziva zdroj proudu pro halla Keithely 6220
#pouziva Agilent data acquisition unit 34970A
#pouziva zdroj proudu pro magnet AgilentN5769A
import time
import sys
import serial

################################
# nastaveni pocatecnich parametru
#jmeno vystupniho souboru

#konfigurace kontaktu
# channels 105 (Hall Voltage),115 (voltage for resistivity) - DC voltage to 5 digits
# channel 101 reading of voltage from teslameter -  magnetic field in Tesla
# channel 102 reading of voltage second sample

SouborHallJmeno="VystupNapetiHall.dat"
SouborOhmJmeno="VystupNapetiOhm.dat"
SouborHall2Jmeno="VystupNapetiHall2.dat"

ProudHall =5.0; # proud vzorky v miliamperech
MagnetProudVal= 1.6  # zadani hodnoty proudu pro magnet v amperech, max 15A
PocatecniTeplota=95  # pocatecni teplota v kelvinech
KonecnaTeplota=490   # koncova teplota
TeplotaKrok=15		#teplotni krok 
TimeStabDelay=90	# doba v sekundach na stabilizaci teploty, typicky mezi 1-100s v zavislosti na vzdalenosti mezi teplotami



# konec nastaveni pocatecnich parametru
################################
CasovaProdlevaPole=7  # casova prodleva kdy se ceka na ustabilizovani mag. pole [v sekundach]
TimeCurrentDelay=1  #doba cekani po nastaveni proudu vzorky kvuli stabilizaci, doporucuji nechat na 1s



# vsechny zakladni inizializace komunikace s pristroji a zakladni rutinky pro komunikaci, uzitecne k nahlednuti
from HallPristrojeInicializace import *

#prikazy pro zapnuti pole plus, minus a vypnuti pole. Vypina u toho zdroj proud pro magnet.
BTimeDelay=1.5  #urcuje casovou mezeru mezi vypinani relatek, kdyz je nula, tak to nefunguje, muze se zrychlit na 0.5s

def BPlusOn():
  dev.digital_on(1) # Turns on digital channel 1
  print "Magneticke pole Plus zapnuto"
  time.sleep(BTimeDelay)
  dev.digital_off(1) # Turns off digital channel 1
  MagnetOn()
  time.sleep(CasovaProdlevaPole)  # cekani na ustabilizovani Pole
  return 


def BOff():
  MagnetOff()
  time.sleep(BTimeDelay)
  dev.digital_on(2) # Turns on digital channel 2
  print "Magneticke pole vypnuto"
  time.sleep(BTimeDelay)
  dev.digital_off(2) # Turns off digital channel 2
  return 


def BMinusOn():
  dev.digital_on(3) # Turns on digital channel 3
  print "Magneticke pole Minus zapnuto"
  time.sleep(BTimeDelay)
  dev.digital_off(3) # Turns off digital channel 3
  MagnetOn()
  time.sleep(CasovaProdlevaPole)  # cekani na ustabilizovani Pole
  return 


# procedura pro nastaveni proudu vzorkem
def NastavProudHall(Proud):
  #print "CURRent "+str(Proud)
  ZdrojProudu("CURRent "+str(Proud)) # nastav proud
  time.sleep(TimeCurrentDelay)  #chvili pocka na ustabilizovani proudu
  return


# procedura pro zmereni napeti na multiplexru (data acquisition unit)
def ZmerNapeti():
  DAU("INIT")  # perform scan
  time.sleep(2)  # musi byt prodleva, v zavislosti na integracni dobe NPLC
  DAU("DATA:REMOVE? 4")    # read data
  mstr=DAUSer.readline()     # konverze dat
  Umag,U2,Ua,Ub=map(float,mstr[:-2:].split(','))
  #print "Namereny string:", mstr
  Ua=Ua*1000 # prepocet na mV
  Ub=Ub*1000 # prepocet na mV
  U2=U2*1000 # prepocet na mV
  Umag=Umag*1000 # prepocet na mT
  print 'U_Hall=%.4G mV\t  U_vodiv=%.4G mV  B=%4.1f mT U_vzorek2= %.4G mV' %(Ua,Ub,Umag,U2) 
  return Ua,Ub,Umag,U2


###########################################################
#hlavni program

ZdrojProudu("*CLS")  # reset instrument
ZdrojProudu(" CURRent:COMPliance 20") # max napeti ve voltech (kompliance)
  
  
#Konfigurace scanu Agilentu Data Acquisition Unit
# channels 105 (Hall Voltage),115 (voltage for resistivity) - DC voltage to 5 digits
# channel 101 reading of voltage from teslameter -  magnetic field in Tesla
# channel 102 reading of voltage second sample
DAU("CONF:VOLT:DC (@105,115)")		# konfigurace kanalu pro cteni halova napeti
DAU("CONF:VOLT:DC (@101)")		# konfigurace kanalu pro napeti z teslametru
DAU("CONF:VOLT:DC (@102)")		# konfigurace kanalu pro cteni napeti z druheho vzorku

#manual: VOLTage:DC:NPLC {0.02|0.2|1|2|10|20|100|200|MIN|MAX}[,(@<ch_list>)]
#DAU("VOLT:NPLC 10")   #5 digits, to improve put larger number
DAU("VOLT:NPLC 20")   # digits, to improve put larger number
DAU("ROUTE:SCAN (@101,102,105,115)")

print "Prirazeni kanalu na Data ackquisition unit Agilentu 34970A"
print "Hallovo napeti - kanal 105, napeti pro odpor - kanal 115"
print "kanal 102 - napeti z druheho vzorku pro mereni odporu"
print "kanal 101 - napeti pro cteni magnetickeho pole"

### vypis nastavenych hodnot:
print
print "Nastavene parametry:"  
print "Proud vzorkem je ", ProudHall, "mA"
ProudHall = ProudHall /1000
MagnetProud(MagnetProudVal)  #  
print "Proud magnetem nastaven na " + str(MagnetProudVal) + "A"
print "Casova prodleva pro cekani na stabilitu pole je " + str(CasovaProdlevaPole) + "s"
PocetTeplot=int(round((KonecnaTeplota-PocatecniTeplota)/TeplotaKrok))
#PocetTeplot=500
print "Pocet mereni:", PocetTeplot
print "Pocatecni teplota: " + str(PocatecniTeplota) + "K"
print "Konecna teplota: " + str(KonecnaTeplota) + "K"
print "Teplotni krok: " + str(TeplotaKrok) + "K/krok"


raw_input("stiskni klavesu")
print


dev.digital_all_off()  # vypne vsechny digitalni vystupy, jen hygyena
BOff()  # vypnuti pole, strasne dulezite, mel bych udelat ochranu pred tim, aby se nekomutovalo bez vypnuteho pole
MagnetNapeti(100)  #nastavi maximalni napeti ve voltech na zdroji pro magnet, max 100V, regulovat se bude proudem



SouborHall = open(SouborHallJmeno,'a')
SouborOhm = open(SouborOhmJmeno,'a')
SouborHall2 = open(SouborHall2Jmeno,'a') #changed
print >> SouborOhm, "T\t B[mT]\t I_Hall\t I_Mag\t UrB0I+\t UrB0I-\t Rave\t UrB+I+\t UrB+I-\t UrB-I+\t UrB-I-\t R_B\t MagResistance"
print >> SouborHall, "T\t B[mT]\t I_Hall\t I_Mag\t UhB+I+\t UhB+I-\t UhB-I+\t UhB-I-\t Uh"

#print >> SouborOhm, "T \t B[mT] \t ProudHall \t ProudMagnet \t UOhm1 \t UOhm1kom \t R \t UOhmPolePlus \t UOhmPolePlusKom \t UOhmPoleMinus \t UOhmPoleMinusKom \t Rpole \t MagnetoResistance"
#print >> SouborHall, "T \t B[mT] \t ProudHall \t ProudMagnet \t UHallPolePlus \t UHallPolePlusKom \t UHallPoleMinus \t UHallPoleMinusKom \t UHall"
print >> SouborHall2, "T\t B[mT]\t I_Hall\t I_Mag\t UhB+I+\t UhB+I-\t UhB-I+\t UhB-I-\t Uh" #TODO - check this
SouborHall.close()
SouborOhm.close()
SouborHall2.close()


#################################
#Merici cyklus

SetPointT=PocatecniTeplota
HeaterOn()
#for SetPointT in xrange(PocatecniTeplota,KonecnaTeplota,TeplotaKrok):
Run=1
while (Run==1):
	SetPoint(SetPointT)
	DAU("VOLTage:DC:RANGe:AUTO ON") #aktualizuje range na AUTO aby pristroj nehazel chybu 222 ("out of range")
	print "Teplotni Setpoint ="+str(SetPointT)+" K"
	print "Cekam " + str(TimeStabDelay) + "s na ustabilizovani teploty"
	time.sleep(TimeStabDelay)   #cekani na stabilizaci teploty
	Tpred=Teplota()  #zmereni teploty pred merenim
	ZdrojProudu(" OUTPut ON") # sepne proud

	#zmer Hallovo napeti kladne pole
	print "Merim pri kladnem poli"
	BPlusOn()   # zapne pole plus
	NastavProudHall(ProudHall)
	UHallPolePlus,UOhmPolePlus,BmerenoPlus,U2Plus=ZmerNapeti()
	NastavProudHall(-ProudHall)
	UHallPolePlusKom,UOhmPolePlusKom,BmerenoPlus,U2PlusKom=ZmerNapeti()
	BOff()  # vypnuti pole, strasne dulezite, mel bych udelat ochranu pred tim, aby se nekomutovalo bez vypnuteho pole

	# zmer vodivost bez magnetickeho pole
	print "Merim pri nulovem poli"
	NastavProudHall(ProudHall)
	UHall1,UOhm1,Bmereno_nula,U2=ZmerNapeti()
	NastavProudHall(-ProudHall)
	UHall1kom,UOhm1kom,Bmereno_nula,U2Kom=ZmerNapeti()
	#prumer hodnot pri opacnem proudu
	R =(UOhm1-UOhm1kom)/2/(ProudHall*1000)
	
	#zmer Hallovo napeti zaporne pole
	print "Merim pri zapornem poli"
	BMinusOn()   # zapne pole minus
	NastavProudHall(ProudHall)
	UHallPoleMinus,UOhmPoleMinus,BmerenoMinus,U2Minus=ZmerNapeti()
	NastavProudHall(-ProudHall)
	UHallPoleMinusKom,UOhmPoleMinusKom,BmerenoMinus,U2MinusKom=ZmerNapeti()
	BOff()  # vypnuti pole, strasne dulezite, mel bych udelat ochranu pred tim, aby se nekomutovalo bez vypnuteho pole

	#Vypocty hodnot
	Bmereno=(BmerenoPlus-BmerenoMinus)/2
	Tpo=Teplota()  #zmereni teploty po mereni
	T=(Tpred+Tpo)/2 #prumerna teplota
	print "Teplota pred merenim " + str(Tpred)+ "K, Teplota po mereni " + str(Tpo) + "K prumerna teplota  "  +str(T)
	Rpole=(UOhmPolePlus - UOhmPolePlusKom + UOhmPoleMinus - UOhmPoleMinusKom)/4/(ProudHall*1000)
	MagnetoResistance =(Rpole-R)/R
	UHall = (UHallPolePlus-UHallPolePlusKom-UHallPoleMinus+UHallPoleMinusKom)/4
	
	#Vypocty hodnot druhy vzorek
	#TODO - remove R2 and R2pole
	R2=(U2-U2Kom)/2/(ProudHall*1000) #not used
	R2pole=(U2Plus-U2PlusKom+U2Minus-U2MinusKom)/4/(ProudHall*1000) #not used
	#MagnetoResistance2=(R2pole-R2)/R2
	MagnetoResistance2=(R2pole-R2)
	UHall2 = (U2Plus-U2PlusKom-U2Minus+U2MinusKom)/4
	# U2, U2Kom,U2Plus,U2PlusKom,U2Minus, U2MinusKom
	
	#ulozeni do souboru
	SouborHall = open(SouborHallJmeno,'a')
	SouborOhm = open(SouborOhmJmeno,'a')
	SouborHall2 = open(SouborHall2Jmeno,'a')
	
	print >> SouborOhm, '%.5G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G' % (T, Bmereno, ProudHall*1000,MagnetProudVal,UOhm1,UOhm1kom,R,UOhmPolePlus,UOhmPolePlusKom,UOhmPoleMinus,UOhmPoleMinusKom,Rpole,MagnetoResistance)
	print >> SouborHall, '%.5G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t' % (T,Bmereno, ProudHall*1000,MagnetProudVal,UHallPolePlus,UHallPolePlusKom,UHallPoleMinus,UHallPoleMinusKom,UHall)
	print >> SouborHall2, '%.5G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t %.4G\t' % (T,Bmereno, ProudHall*1000,MagnetProudVal,U2Plus,U2PlusKom,U2Minus, U2MinusKom,UHall2)

	#print >> SouborOhm, '%5.2f %4.1f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f' % (T, Bmereno, ProudHall*1000,MagnetProudVal,UOhm1,UOhm1kom,R,UOhmPolePlus,UOhmPolePlusKom,UOhmPoleMinus,UOhmPoleMinusKom,Rpole,MagnetoResistance)
	#print >> SouborHall, ' %5.2f %4.1f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f' % (T,Bmereno, ProudHall*1000,MagnetProudVal,UHallPolePlus,UHallPolePlusKom,UHallPoleMinus,UHallPoleMinusKom,UHall)
	#print >> Soubor2, ' %5.2f %4.1f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f %15.5f ' % (T,Bmereno, ProudHall*1000,MagnetProudVal,U2, U2Kom,U2Plus,U2PlusKom,U2Minus, U2MinusKom,R2,R2pole,MagnetoResistance2)
	SouborHall.close()
	SouborOhm.close()
	SouborHall2.close()
	print
	print "Konec mereni jedne teploty"
	SetPointT=SetPointT+TeplotaKrok
	#print "Konec mereni jedne teploty, cekam 10s, mozno prerusit"
	print
	#if statement pro ukonceni mereni pokud je soucasna (ne pozadovana) teplota za pozadovanym rozsahem. Umoznuje mereni jak nahoru tak dolu v teplote
	if (KonecnaTeplota > PocatecniTeplota):  #pri rustu teploty
		if (T>KonecnaTeplota):
			Run=0;
	elif (KonecnaTeplota <= PocatecniTeplota):	#pri klesani teploty
		if (T<KonecnaTeplota):
			Run=0;


################################
#konec mericiho cyklu

HeaterOff()
ZdrojProudu(" OUTPut OFF") # vypne proud
#raw_input('proud je vypnuty ')





