#Program na  mereni Hallova napeti pro ruzne hodnoty magnetickeho pole, tedy pro stestovani podminky slabeho pole
#pouziva zdroj proudu pro halla Keithely 6220
#pouziva Agilent data acquisition unit 34970A
#pouziva zdroj proudu pro magnet AgilentN5769A


################################
# nastaveni pocatecnich parametru
#konfigurace kontaktu
# channels 105 (sample Voltage) - DC voltage to 5 digits  -  voltage switched by the comutator OB-11-K
# channel 101 reading of voltage from teslameter -  magnetic field in Tesla

SouborHallJmeno="VystupNapetiHall.dat"
SouborOhmJmeno="VystupNapetiOhm.dat"
#SouborHall2Jmeno="VystupNapetiHall2.dat"

ProudHall =5.0; # proud vzorky v miliamperech
MagnetProudVal= 1.6  # zadani hodnoty proudu pro magnet v amperech, max 15A
PocatecniTeplota=296  # pocatecni teplota v kelvinech
KonecnaTeplota=460   # koncova teplota
TeplotaKrok=1		#teplotni krok 
TimeStabDelay=5	# 90 doba v sekundach na stabilizaci teploty, typicky mezi 1-100s v zavislosti na vzdalenosti mezi teplotami
CasovaProdlevaPole=7  # casova prodleva kdy se ceka na ustabilizovani mag. pole [v sekundach]
TimeCurrentDelay=1  #doba cekani po nastaveni proudu vzorky kvuli stabilizaci, doporucuji nechat na 1s

# konec nastaveni pocatecnich parametru
################################


import time
import sys
import serial

# vsechny zakladni inizializace komunikace s pristroji a zakladni rutinky pro komunikaci, uzitecne k nahlednuti
from HallPristrojeInicializace import *

#prikazy pro zapnuti pole plus, minus a vypnuti pole. Vypina u toho zdroj proud pro magnet.
BTimeDelay=1  #urcuje casovou mezeru mezi vypinani relatek, kdyz je nula, tak to nefunguje, muze se zrychlit na 0.5s

def BPlusOn():
  dev.digital_on(1) # Turns on digital channel 1
  print "Magneticke pole Plus zapnuto"
  time.sleep(BTimeDelay)
  dev.digital_off(1) # Turns off digital channel 1
  MagnetOn()
  time.sleep(CasovaProdlevaPole)  # cekani na ustabilizovani Pole
  return 


def BOff():
  MagnetOff()
  time.sleep(BTimeDelay)
  dev.digital_on(2) # Turns on digital channel 2
  print "Magneticke pole vypnuto"
  time.sleep(BTimeDelay)
  dev.digital_off(2) # Turns off digital channel 2
  return 


def BMinusOn():
  dev.digital_on(3) # Turns on digital channel 3
  print "Magneticke pole Minus zapnuto"
  time.sleep(BTimeDelay)
  dev.digital_off(3) # Turns off digital channel 3
  MagnetOn()
  time.sleep(CasovaProdlevaPole)  # cekani na ustabilizovani Pole
  return 


# procedura pro nastaveni proudu vzorkem
def NastavProudHall(Proud):
  #print "CURRent "+str(Proud)
  ZdrojProudu("CURRent "+str(Proud)) # nastav proud
  time.sleep(TimeCurrentDelay)  #chvili pocka na ustabilizovani proudu
  return


  
def ZmerU():
  DAU("INIT")  # perform scan
  time.sleep(2)  # musi byt prodleva, v zavislosti na integracni dobe NPLC
  DAU("DATA:REMOVE? 2")    # read data
  mstr=DAUSer.readline()     # konverze dat
  Umag,U=map(float,mstr[:-2:].split(','))
  #print "Namereny string:", mstr
  U=U*1000 # prepocet na mV
  Umag=Umag*1000 # prepocet na mT
  return U,Umag
  

def D(cislo):
	return '{:.4G}'.format(cislo)   # dekadicky format, ctyri platne cislice, 
  
  
# procedura pro zmereni napeti na multiplexru (data acquisition unit)
def ZmerNapetiKomut(komut, ProudHall):
  Komutator(komut)
  time.sleep(0.2)
  NastavProudHall(ProudHall)   #mereni pri kladnem proudu
  U1,Umag1=ZmerU()
  NastavProudHall(-ProudHall)   #mereni pri zapornem proudu
  U2,Umag2=ZmerU()
  #ZdrojProudu(" OUTPut OFF") # vypne proud vzorkem
  NastavProudHall(0)   #nastavi nulovy proud, aby komutace netrapila zdroj
  #print 'Napeti =%8.4f mV,  B =%4.1f mT' %(U,Umag)   
  B=(Umag1+Umag2)/2
  print "Komutace=",komut," U1=", D(U1), " mV", " U2=", D(U2), " mV", "B=", D(B), " mT"
  return U1,U2,B

  
def f(cislo):
	#return '{:.4G}'.format(cislo)   # dekadicky format, ctyri platne cislice, 
	return '{:.4e}'.format(cislo)   # vedecka notace, ctyri cisla za desetinnou carkou
	


###########################################################
#hlavni program


#Konfigurace scanu Agilentu Data Acquisition Unit
# channels 105 (Hall Voltage) - DC voltage to 5 digits
# channel 101 reading of voltage from teslameter -  magnetic field in Tesla
DAU("CONF:VOLT:DC (@105)")		# konfigurace kanalu pro cteni halova napeti
DAU("CONF:VOLT:DC (@101)")		# konfigurace kanalu pro napeti z teslametru
#manual: VOLTage:DC:NPLC {0.02|0.2|1|2|10|20|100|200|MIN|MAX}[,(@<ch_list>)]
#DAU("VOLT:NPLC 10")   #5 digits, to improve put larger number
DAU("VOLT:NPLC 20")   # digits, to improve put larger number
DAU("ROUTE:SCAN (@101,105)")


dev.digital_all_off()  # vypne vsechny digitalni vystupy, jen hygyena, nevim proc nefunguje
BOff   # vypne zdroj proudu pro magnet a vypne digitalni vystup 2
MagnetNapeti(100)  #nastavi maximalni napeti ve voltech na zdroji pro magnet, max 100V, regulovat se bude proudem
ZdrojProudu("*CLS")  # reset instrument Keithley 6220
ZdrojProudu(" CURRent:COMPliance 20") # max napeti ve voltech (kompliance)

print
print
print "Nastavene parametry:"  
print "Proud vzorkem je ", ProudHall, "mA"
print "Teplota vzorku ", PocatecniTeplota, " K"

MagnetProud(MagnetProudVal)  #  
print "Proud magnetem nastaven na " + str(MagnetProudVal) + "A"
print "Casova prodleva pro cekani na stabilitu pole je " + str(CasovaProdlevaPole) + "s"
PocetTeplot=int(round((KonecnaTeplota-PocatecniTeplota)/TeplotaKrok))
#PocetTeplot=500
print "Pocet mereni:", PocetTeplot
print "Pocatecni teplota: " + str(PocatecniTeplota) + "K"
print "Konecna teplota: " + str(KonecnaTeplota) + "K"
print "Teplotni krok: " + str(TeplotaKrok) + "K/krok"

ProudHall = ProudHall /1000

BOff()  # vypnuti pole, strasne dulezite, mel bych udelat ochranu pred tim, aby se nekomutovalo bez vypnuteho pole
raw_input("stiskni klavesu")
print

#Hlavicka vystupnich souboru
SouborHall = open(SouborHallJmeno,'a')
SouborOhm = open(SouborOhmJmeno,'a')
print >> SouborHall, "T[K]    ", "\t", "B[mT]    ", "\t", "Ih[mA]  ", \
			"\t","Uh1p[mV]" ,"\t", "Uh2p[mV]","\t", "Uh1m[mV]" ,"\t","Uh2m[mV]", "\t","Uh_h[mV]",\
			"\t", "Uk1p[mV]", "\t","Uk2p[mV]", "\t","Uk1m[mV]","\t","Uk2m[mV]", "\t", "Uh_k[mV]" 

			
print >> SouborOhm, "T[K]    ",  "\t", "B[mT]  ",  "\t", "Imag[A]" , "\t", "Ih[mA]  " ,  \
			"\t", "Ua1p[mV]", "\t", "Ua2p[mV]", "\t", "Ua1m[mV]", "\t", "Ua2m[mV]", "\t", "Ua1z[mV]", "\t", "Ua2z[mV]", "\t", "R_a[Ohm]",  "\t", "MagR_a  ",\
			"\t", "Ub1p[mV]", "\t", "Ub2p[mV]", "\t", "Ub1m[mV]", "\t", "Ub2m[mV]", "\t", "Ub1z[mV]",  "\t", "Ub2z[mV]", "\t", "R_b[Ohm]",  "\t", "MagR_b  " 
SouborHall.close()
SouborOhm.close()

NastavProudHall(0)
ZdrojProudu(" OUTPut ON") # sepne proud vzorkem


#################################
#Merici cyklus
SetPointT=PocatecniTeplota
HeaterOn()
Run=1
while (Run==1):
	#nastaveni teploty
	SetPoint(SetPointT)
	#DAU("VOLTage:DC:RANGe:AUTO ON") #aktualizuje range na AUTO aby pristroj nehazel chybu 222 ("out of range")
	print "Teplotni Setpoint ="+str(SetPointT)+" K"
	print "Cekam " + str(TimeStabDelay) + "s na ustabilizovani teploty"
	time.sleep(TimeStabDelay)
	Tpred=Teplota()  #zmereni teploty pred merenim

	#zmer  napeti pro kladne pole
	print "Merim pri kladnem poli"
	BPlusOn()   # zapne pole plus
	Uh1p,Uh2p,BPlus=ZmerNapetiKomut('h', ProudHall)
	Uk1p,Uk2p,BPlus=ZmerNapetiKomut('k', ProudHall)
	Ua1p,Ua2p,BPlus=ZmerNapetiKomut('a', ProudHall)
	Ub1p,Ub2p,BPlus=ZmerNapetiKomut('b', ProudHall)
	BOff()  # vypnuti pole, strasne dulezite, mel bych udelat ochranu pred tim, aby se nekomutovalo bez vypnuteho pole

	# zmer vodivost bez magnetickeho pole
	print "Merim pri nulovem poli"
	Ua1z,Ua2z,Bnula=ZmerNapetiKomut('a', ProudHall)
	Ub1z,Ub2z,Bnula=ZmerNapetiKomut('b', ProudHall)
		
	#zmer Hallovo napeti zaporne pole
	print "Merim pri zapornem poli"
	BMinusOn()   # zapne pole minus
	Uh1m,Uh2m,BMinus=ZmerNapetiKomut('h', ProudHall)
	Uk1m,Uk2m,BMinus=ZmerNapetiKomut('k', ProudHall)
	Ua1m,Ua2m,BMinus=ZmerNapetiKomut('a', ProudHall)
	Ub1m,Ub2m,BMinus=ZmerNapetiKomut('b', ProudHall)
	BOff()  # vypnuti pole, strasne dulezite, mel bych udelat ochranu pred tim, aby se nekomutovalo bez vypnuteho pole

	Bmereno=(BPlus-BMinus)/2
	Tpo=Teplota()  #zmereni teploty po mereni
	T=(Tpred+Tpo)/2 #prumerna teplota
	print "pred merenim T=" + str(Tpred)+ "K, po mereni T=" + str(Tpo) + "K prumerna T="  +str(T)

	#vypocet Hallova napeti, odporu a magnetorezistence 
	Uhall_h = (Uh1p-Uh2p-Uh1m+Uh2m)/4
	Uhall_k = (Uk1p-Uk2p-Uk1m+Uk2m)/4
	print "U Hall h=", D(Uhall_h)
	print "U Hall k=", D(Uhall_k)
	UBa= (Ua1p-Ua2p+Ua1m-Ua2m)/4
	Ua0=(Ua1z-Ua2z)/2
	MagR_a=(UBa-Ua0)/Ua0
	Ra=Ua0/(ProudHall*1000)
	UBb= (Ub1p-Ub2p+Ub1m-Ub2m)/4
	Ub0=(Ub1z-Ub2z)/2
	MagR_b=(UBb-Ub0)/Ub0
	Rb=Ub0/(ProudHall*1000)
	print "Odpor Ra=", D(Ra), "Ohm", "Magnetoresistance a=", D(MagR_a)
	print "Odpor Rb=", D(Rb), "Ohm", "Magnetoresistance b=", D(MagR_b)

	#ulozeni do souboru
	SouborHall = open(SouborHallJmeno,'a')
	SouborOhm = open(SouborOhmJmeno,'a')
	print >> SouborHall, f(T), "\t", f(Bmereno), "\t", f(ProudHall*1000), \
			"\t", f(Uh1p),"\t", f(Uh2p),"\t", f(Uh1m),"\t",f(Uh2m), "\t", f(Uhall_h), \
			"\t",f(Uk1p),"\t",f(Uk2p),"\t", f(Uk1m),"\t", f(Uk2m), "\t",f(Uhall_k) 
			
	print >> SouborOhm, f(T), "\t", f(Bmereno), "\t", f(MagnetProudVal), "\t",f(ProudHall*1000),  \
			"\t", f(Ua1p), "\t", f(Ua2p), "\t", f(Ua1m), "\t", f(Ua2m), "\t", f(Ua1z), "\t", f(Ua2z), "\t", f(Ra) , "\t", f(MagR_a), \
			"\t", f(Ub1p), "\t", f(Ub2p), "\t", f(Ub1m), "\t", f(Ub2m), "\t", f(Ub1z), "\t", f(Ub2z), "\t", f(Rb) ,	"\t", f(MagR_b)
	SouborHall.close()
	SouborOhm.close()

	print
	print "Konec mereni jedne teploty"
	SetPointT=SetPointT+TeplotaKrok
	#print "Konec mereni jedne teploty, cekam 10s, mozno prerusit"
	print
	#if statement pro ukonceni mereni pokud je soucasna (ne pozadovana) teplota za pozadovanym rozsahem. Umoznuje mereni jak nahoru tak dolu v teplote
	if (KonecnaTeplota > PocatecniTeplota):  #pri rustu teploty
		if (T>KonecnaTeplota):
			Run=0;
	elif (KonecnaTeplota <= PocatecniTeplota):	#pri klesani teploty
		if (T<KonecnaTeplota):
			Run=0;


################################
#konec mericiho cyklu

SetPointT=80
SetPoint(SetPointT)

ZdrojProudu(" OUTPut OFF") # vypne proud
#raw_input('proud je vypnuty ')





