import numpy as np
#import matplotlib.pyplot as plt
from lmfit import Minimizer, Parameters, fit_report
import pylab

# cas v sekundach
#t = np.arange(1E-7, 100E-6, 1E-7)

#reseni difusni rovnice, Brownian motion
def rho(t,x,D,tau,mu,E,N,bckg,t0):
	return N/(4*np.pi*D*(t-t0))**(1/2)*np.exp(-(t-t0)/tau-(x-E*mu*(t-t0))**2/(4*D*(t-t0)))+bckg



class AllData:   #definition of class of all data, spectra, positions, voltages
	t=[]	
	Spectrum=[]
	U=[]
	x=[]
	l =0   # delka vzorku [cm]




params = Parameters()
params.add('t0', value=5E-6,vary=False)   # cas 0, asi polosirka pulzu
params.add('N', value=0.0013,vary=True)   # amplituda
#params.add('bckg', value=7.8E-4,vary=True)   # pozadi
params.add('D', value=49,vary=True)    #difusni konstanta v cm^2/s
params.add('tau', value=130E-6,vary=True)
params.add('mu', value=1471,vary=True)




# hodnoty k optimalizaci
StartLine=330
EndLine=1600
AllData.l=1.4   # delka vzorku [cm]
# hodnoty k optimalizaci
PocetSpekter=0



data=np.loadtxt('scope_64_4V.csv', skiprows=2,delimiter=",") 
AllData.t.append(data[StartLine:EndLine,0])
AllData.Spectrum.append(data[StartLine:EndLine,1])
AllData.U.append(4)   #voltage in V
AllData.x.append(0.2)  #position in cm
params.add('bckg'+str(PocetSpekter), value=7.8E-4,vary=True)   # pozadi
PocetSpekter=PocetSpekter+1


data=np.loadtxt('scope_63_3V.csv', skiprows=2,delimiter=",") 
AllData.t.append(data[StartLine:EndLine,0])
AllData.Spectrum.append(data[StartLine:EndLine,1])
AllData.U.append(3)   #voltage in V
AllData.x.append(0.2)  #position in cm
params.add('bckg'+str(PocetSpekter), value=7.8E-4,vary=True)   # pozadi
PocetSpekter=PocetSpekter+1

'''
data=np.loadtxt('scope_61_2V.csv', skiprows=2,delimiter=",") 
AllData.t.append(data[StartLine:EndLine,0])
AllData.Spectrum.append(data[StartLine:EndLine,1])
AllData.U.append(2)   #voltage in V
AllData.x.append(0.2)  #position in cm
params.add('bckg'+str(PocetSpekter), value=7.8E-4,vary=True)   # pozadi
PocetSpekter=PocetSpekter+1
'''


data=np.loadtxt('scope_62_1V.csv', skiprows=2,delimiter=",") 
#StartLine=400
#EndLine=1600
AllData.t.append(data[StartLine:EndLine,0])
AllData.Spectrum.append(data[StartLine:EndLine,1])
AllData.U.append(1)   #voltage in V
AllData.x.append(0.2)  #position in cm
params.add('bckg'+str(PocetSpekter), value=7.8E-4,vary=True)   # pozadi
PocetSpekter=PocetSpekter+1



print("Pocet spekter=",len(AllData.t))


def EvalRho(p,AllData,n):
	ElField=AllData.U[n]/AllData.l
	return rho(AllData.t[n],AllData.x[n],p['D'],p['tau'],p['mu'],ElField,p['N'],p['bckg'+str(n)],p['t0'])

#definice residualu: rozdilu fitovane funkce a dat
def residual(p, AllData):
	res=0
	for n in range(len(AllData.t)):
		#print("n=",n)
		#print(AllData.x[n])
		#pylab.plot(AllData.t[n], AllData.Spectrum[n], 'k-', label='Data')
		#pylab.show()
		res=res+ pow(EvalRho(p,AllData,n)-AllData.Spectrum[n],2)
	return pow(res,0.5)



#fitovani
minner = Minimizer(residual, params, fcn_args=(AllData,))   #ta posledni carka tam musi byt, pokud tech parametru neni vice.
result = minner.minimize()
ResultingP= result.params.valuesdict()
print(fit_report(result))  # write error report, alternative result.params.pretty_print()


#ulozeni statistiky do souboru
FileFit = open('Statistika.dat', 'w+')
print(fit_report(result),file=FileFit)
FileFit.close()


#ulozeni statistiky do souboru
FileVystup = open('Vystup.dat', 'w+')

for i in range(len(AllData.t[0])):
	ToFile=str(AllData.t[0][i])
	for n in range(len(AllData.t)):
		ToFile=ToFile+	"\t" + str(AllData.Spectrum[n][i])  +"\t" + str(EvalRho(ResultingP,AllData,n)[i])
	print(ToFile,file=FileVystup)

FileVystup.close()


# vykresleni fitu na monitor
for n in range(len(AllData.t)):
	pylab.plot(AllData.t[n], AllData.Spectrum[n], 'k-', label='Data')
	pylab.plot(AllData.t[n], EvalRho(ResultingP,AllData,n), 'r-',label='Nafitovany model')
	#pylab.plot(AllData.t[n],  EvalRho(params,AllData,n), 'b--',label='Puvodni model')
pylab.ticklabel_format(style='sci', axis='x', scilimits=(0,0))
pylab.legend(loc='upper right', shadow=False)
pylab.show()
pylab.savefig('Vystup.png')



